@@ -9,6 +9,9 @@ from datetime import datetime
 # Apple Epoch is Jan 1, 2001
 TIME_DELTA = (datetime(2001, 1, 1, 0, 0) - datetime(1970, 1, 1, 0, 0)).total_seconds()
 
+# Unicode format to use for comparing strings
+UNICODE_FORMAT = "NFC"
+
 # which Photos library database versions have been tested
 # Photos 2.0 (10.12.6) == 2622
 # Photos 3.0 (10.13.6) == 3301
@@ -43,6 +46,7 @@ _DB_TABLE_NAMES = {
         "ALBUM_JOIN": "Z_26ASSETS.Z_34ASSETS",
         "ALBUM_SORT_ORDER": "Z_26ASSETS.Z_FOK_34ASSETS",
         "IMPORT_FOK": "ZGENERICASSET.Z_FOK_IMPORTSESSION",
+        "DEPTH_STATE": "ZGENERICASSET.ZDEPTHSTATES",
     },
     6: {
         "ASSET": "ZASSET",
@@ -50,6 +54,7 @@ _DB_TABLE_NAMES = {
         "ALBUM_JOIN": "Z_26ASSETS.Z_3ASSETS",
         "ALBUM_SORT_ORDER": "Z_26ASSETS.Z_FOK_3ASSETS",
         "IMPORT_FOK": "null",
+        "DEPTH_STATE": "ZASSET.ZDEPTHTYPE",
     },
 }
 
